#!/bin/bash

############### Author: Emad Aghajani
############### What does this script do?
###############		Given a list of repos (as owner/repo) in "repos_list", clone them to "clone_to"
###############

# Define a timestamp function
timestamp() {
  date +"%D-%T"
}

repos_list=$1
clone_to="./repos"

nTotal=0
nSuccess=0
nFail=0

log_output="${PWD}/clone_out.log"
log_error="${PWD}/clone_error.log"
rm -f $log_output
rm -f $log_error


mkdir -p $clone_to


while IFS= read -r owner_repo ############### For each item in "repos_list"
do
	nTotal=$((nTotal+1))

	############### Break each item to "owner" and "repo"
	owner=`dirname "$owner_repo"`
	repo=`basename "$owner_repo"`

	echo -e "$(timestamp)\t$nTotal $owner/$repo" >> "$log_output"

	mkdir -p "$clone_to/$owner"

	pushd "$clone_to/$owner"  > /dev/null

	if [ -d $repo ]; then ############### if repo already exists, delete it
		echo -e "$(timestamp)\tRemoving old $repo at $PWD" >> "$log_output"
		rm -rf $repo
	fi

	############### Clone
	# GIT_TERMINAL_PROMPT=0: will not prompt for authentication. See https://git-scm.com/docs/git#GIT_TERMINAL_PROMPT
	# Below line is correct and doen't need && before "git clone ..."
	GIT_TERMINAL_PROMPT=0 git clone --single-branch --quiet "https://github.com/$owner/$repo" 1>>"$log_output"  2>>"$log_error"
	# only primary branch: --single-branch
	# with submodules: --recurse-submodules --shallow-submodules

	############### Check clone result
	res=$?
	if [ $res -eq 0 ]; then
	    echo -e "$(timestamp)\t$nTotal $owner/$repo: [SUCCESS]"
	    echo -e "$(timestamp)\tCloned successfully" >> "$log_output"
	    nSuccess=$((nSuccess+1))
	else
	    echo -e "$(timestamp)\t$nTotal $owner/$repo: [FAIL] - $res"
	    echo -e "$(timestamp)\tCloned failed - return code: $res" >> "$log_output"
	    nFail=$((nFail+1))
	    
	    if [ -z "$(ls -A -- ./)" ]; then # if cur dir (=owner) is empty
	    	## If from this "owner" no other repo cloned before, delete empty 'owner" dir
	    	cd ..
	    	rm -rf "$owner"
	    fi
	fi

	popd > /dev/null

	echo "" >> "$log_output"
	echo "" >> "$log_error"

done < "${repos_list}"


echo "------------------------------------------"
echo "#Repo Total : $nTotal"
echo "#Repo Clone Successful: $nSuccess"
echo "#Repo Clone Failed: $nFail"
echo "------------------------------------------$(timestamp)"
